// ignore_for_file: deprecated_member_use
// ignore_for_file: prefer_collection_literals

import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'dart:convert';

import '../models/ortho_model.dart';

class OrthoWeb extends StatefulWidget {
  ///config params for ortho
  final Map<String, dynamic>? config;

  ///the slug is the payment sdk you choose to use
  final String? slug;
  const OrthoWeb({
    Key? key,
    this.config,
    this.slug,
  })  : assert(config != null),
        super(key: key);

  @override
  State<OrthoWeb> createState() => _OrthoWebState();
}

class _OrthoWebState extends State<OrthoWeb> {
  ///initialiaze webview controller
  WebViewController? _controller;

  ///call orthoModel to handle transactions
  OrthoModel orthoModel = OrthoModel(false, false, false, false, "");

  static final Dio _dio = Dio(BaseOptions(
      baseUrl: "https://api.tryortho.co/v2/sdk/boot-json/ortho-flow"));

  ///Start a transaction
  ///
  ///Makes an api call with ortho options
  ///Gets the html response to be used in a webview
  Future initTransaction() async {
    ///convert the json to a string
    var configString = jsonEncode(widget.config);
    try {
      Response response =
          await _dio.get("/${widget.slug}/$configString/flutter");

      return response.data['body'];
    } on DioError catch (e) {
      debugPrint(
          "FlutterOrtho : Error initializing transaction: ${e.response!.data['message']}");
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      bottomNavigationBar: Container(
        height: MediaQuery.of(context).size.height / 19,
        width: MediaQuery.of(context).size.width,
        color: Colors.blue,
        child: Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: [
            TextButton(
                child:
                    const Text('Close', style: TextStyle(color: Colors.white)),
                onPressed: () {
                  Navigator.pop(context);
                }),
          ],
        ),
      ),
      body: Stack(
        children: [
          FutureBuilder(
            future: initTransaction(),
            builder: (context, snapshot) {
              if (snapshot.hasData) {
                debugPrint(snapshot.data.toString());
                return WebView(
                    onPageFinished: (response) {
                      debugPrint(response);
                    },
                    javascriptMode: JavascriptMode.unrestricted,
                    gestureRecognizers: [
                      Factory(() => VerticalDragGestureRecognizer()),
                      Factory(() => TapGestureRecognizer())
                    ].toSet(),
                    javascriptChannels: Set.from([
                      JavascriptChannel(
                          name: 'messageHandler',
                          onMessageReceived: (JavascriptMessage message) {
                            orthoModel = OrthoModel(
                                true, true, false, false, message.message);
                            debugPrint(orthoModel.data);
                          }),
                    ]),
                    onWebViewCreated: (webViewController) {
                      _controller = webViewController;
                      _controller?.loadHtmlString(snapshot.data.toString());
                    });
              } else if (snapshot.hasError) {
                return const Center(
                  child: Text(
                    'An error occured, please close the modal and try again',
                    style: TextStyle(color: Colors.black),
                  ),
                );
              }
              return const Center(
                  child: CircularProgressIndicator(color: Colors.black));
            },
          ),
        ],
      ),
      //  bottomSheet:
    );
  }
}
